@@ -53,10 +53,6 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 - (void) _setupWindowNotifications;
 @end
 
-@interface TKApplication(TKScrlbr)
-- (void) _setupScrollBarNotifications;
-@end
-
 @interface TKApplication(TKMenus)
 - (void) _setupMenus;
 @end
@@ -91,11 +87,9 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 
 - (void) _setupEventLoop
 {
-    _running = 1;
-    if (!_appFlags._hasBeenRun) {
-        _appFlags._hasBeenRun = YES;
-	[self finishLaunching];
-    }
+
+    /*Remove private API flags here.*/
+    [self finishLaunching];
     [self setWindowsNeedUpdate:YES];
 }
 
@@ -110,7 +104,6 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 	    selector:@selector(_postedNotification:) name:nil object:nil];
 #endif
     [self _setupWindowNotifications];
-    [self _setupScrollBarNotifications];
     [self _setupApplicationNotifications];
 }
 
@@ -236,12 +229,17 @@ TkpInit(
 	if (!uname(&name)) {
 	    tkMacOSXMacOSXVersion = (strtod(name.release, NULL) + 96) * 10;
 	}
-	if (tkMacOSXMacOSXVersion &&
+       /*Check for new versioning scheme on Yosemite (10.10) and later.*/
+	if (MAC_OS_X_VERSION_MIN_REQUIRED > 100000) {
+		tkMacOSXMacOSXVersion = MAC_OS_X_VERSION_MIN_REQUIRED/100;
+	    }
+	if (tkMacOSXMacOSXVersion && MAC_OS_X_VERSION_MIN_REQUIRED < 100000 &&
 		tkMacOSXMacOSXVersion/10 < MAC_OS_X_VERSION_MIN_REQUIRED/10) {
 	    Tcl_Panic("Mac OS X 10.%d or later required !",
 		    (MAC_OS_X_VERSION_MIN_REQUIRED/10)-100);
 	}
 
+
 #ifdef TK_FRAMEWORK
 	/*
 	 * When Tk is in a framework, force tcl_findLibrary to look in the
