@@ -7,7 +7,6 @@
  * Copyright 2001-2009, Apple Inc.
  * Copyright (c) 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
  * Copyright (c) 2012 Adrian Robert.
- * Copyright 2015 Marc Culler. 
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -28,9 +27,7 @@ static Tk_Window grabWinPtr = NULL;
 				/* Current grab window, NULL if no grab. */
 static Tk_Window keyboardGrabWinPtr = NULL;
 				/* Current keyboard grab window. */
-static NSWindow *keyboardGrabNSWindow = nil;
-                               /* NSWindow for the current keyboard grab window. */
-static NSModalSession modalSession = nil;
+static NSModalSession modalSession = NULL;
 
 static BOOL processingCompose = NO;
 static BOOL finishedCompose = NO;
@@ -81,18 +78,16 @@ static unsigned isFunctionKey(unsigned int code);
     case NSFlagsChanged:
 	modifiers = [theEvent modifierFlags];
 	keyCode = [theEvent keyCode];
-	//	w = [self windowWithWindowNumber:[theEvent windowNumber]];
-	w = [theEvent window];
+	w = [self windowWithWindowNumber:[theEvent windowNumber]];
 #if defined(TK_MAC_DEBUG_EVENTS) || NS_KEYLOG == 1
 	NSLog(@"-[%@(%p) %s] r=%d mods=%u '%@' '%@' code=%u c=%d %@ %d", [self class], self, _cmd, repeat, modifiers, characters, charactersIgnoringModifiers, keyCode,([charactersIgnoringModifiers length] == 0) ? 0 : [charactersIgnoringModifiers characterAtIndex: 0], w, type);
 #endif
 	break;
 
     default:
-	return theEvent; /* Unrecognized key event. */
+	return theEvent;
     }
 
-    /* Create an Xevent to add to the Tk queue. */
     if (!processingCompose) {
         unsigned int state = 0;
 
@@ -133,7 +128,7 @@ static unsigned isFunctionKey(unsigned int code);
         tkwin = (Tk_Window) winPtr->dispPtr->focusPtr;
         if (!tkwin) {
           TkMacOSXDbgMsg("tkwin == NULL");
-          return theEvent;  /* Give up. No window for this event. */
+          return theEvent;
         }
 
         /*
@@ -227,7 +222,7 @@ static unsigned isFunctionKey(unsigned int code);
 
 
 
-@implementation TKContentView
+@implementation TKContentView(TKKeyEvent)
 /* <NSTextInput> implementation (called through interpretKeyEvents:]). */
 
 /* <NSTextInput>: called when done composing;
@@ -293,6 +288,22 @@ static unsigned isFunctionKey(unsigned int code);
 }
 
 
+/* delete display of composing characters [not in <NSTextInput>] */
+- (void)deleteWorkingText
+{
+  if (privateWorkingText == nil)
+    return;
+  if (NS_KEYLOG)
+    NSLog(@"deleteWorkingText len = %lu\n",
+	    (unsigned long)[privateWorkingText length]);
+  [privateWorkingText release];
+  privateWorkingText = nil;
+  processingCompose = NO;
+
+  //PENDING: delete working text
+}
+
+
 - (BOOL)hasMarkedText
 {
   return privateWorkingText != nil;
@@ -328,7 +339,7 @@ static unsigned isFunctionKey(unsigned int code);
   pt.y = caret_y;
 
   pt = [self convertPoint: pt toView: nil];
-  pt = [[self window] convertPointToScreen: pt];
+  pt = [[self window] convertBaseToScreen: pt];
   pt.y -= caret_height;
 
   rect.origin = pt;
@@ -402,24 +413,6 @@ static unsigned isFunctionKey(unsigned int code);
 @end
 
 
-@implementation TKContentView(TKKeyEvent)
-/* delete display of composing characters [not in <NSTextInput>] */
-- (void)deleteWorkingText
-{
-  if (privateWorkingText == nil)
-    return;
-  if (NS_KEYLOG)
-    NSLog(@"deleteWorkingText len = %lu\n",
-	    (unsigned long)[privateWorkingText length]);
-  [privateWorkingText release];
-  privateWorkingText = nil;
-  processingCompose = NO;
-
-  //PENDING: delete working text
-}
-@end
-
-
 
 /*
  *  Set up basic fields in xevent for keyboard input.
@@ -481,9 +474,7 @@ XGrabKeyboard(
 	    if (modalSession) {
 		Tcl_Panic("XGrabKeyboard: already grabbed");
 	    }
-	    keyboardGrabNSWindow = w;
-	    [w retain];
-	    modalSession = [NSApp beginModalSessionForWindow:w];
+	    modalSession = [NSApp beginModalSessionForWindow:[w retain]];
 	}
     }
     return GrabSuccess;
@@ -511,12 +502,11 @@ XUngrabKeyboard(
     Time time)
 {
     if (modalSession) {
+	NSWindow *w = keyboardGrabWinPtr ? TkMacOSXDrawableWindow(
+		((TkWindow *) keyboardGrabWinPtr)->window) : nil;
 	[NSApp endModalSession:modalSession];
-	modalSession = nil;
-    }
-    if (keyboardGrabNSWindow) {
-	[keyboardGrabNSWindow release];
-	keyboardGrabNSWindow = nil;
+	[w release];
+	modalSession = NULL;
     }
     keyboardGrabWinPtr = NULL;
 }
