@@ -258,10 +258,9 @@ static int	ModifierCharWidth(Tk_Font tkfont);
 
 	if (menuPtr && mePtr) {
 	    Tcl_Interp *interp = menuPtr->interp;
-	    /*Add time for errors to fire if necessary. This is sub-optimal
-	     *but avoids issues with Tcl/Cocoa event loop integration.
-	     */
+	    /*Add time for errors to fire if necessary. This is sub-optimal but avoids issues with Tcl/Cocoa event loop integration.*/
 	    Tcl_Sleep(100);
+
 	    Tcl_Preserve(interp);
 	    Tcl_Preserve(menuPtr);
 
@@ -270,7 +269,7 @@ static int	ModifierCharWidth(Tk_Font tkfont);
 	    if (result != TCL_OK && result != TCL_CONTINUE &&
 		    result != TCL_BREAK) {
 		Tcl_AddErrorInfo(interp, "\n    (menu invoke)");
-		Tcl_BackgroundError(interp);
+		Tcl_BackgroundException(interp, result);
 	    }
 	    Tcl_Release(menuPtr);
 	    Tcl_Release(interp);
@@ -360,7 +359,7 @@ static int	ModifierCharWidth(Tk_Font tkfont);
 
 	if (result!=TCL_OK && result!=TCL_CONTINUE && result!=TCL_BREAK) {
 	      Tcl_AddErrorInfo(interp, "\n    (menu preprocess)");
-	      Tcl_BackgroundError(interp);
+	      Tcl_BackgroundException(interp, result);
 	}
 	Tcl_Release(menuPtr);
 	Tcl_Release(interp);
@@ -412,7 +411,7 @@ static int	ModifierCharWidth(Tk_Font tkfont);
 
 	    if (!mePtr || !(mePtr->entryFlags & ENTRY_APPLE_MENU)) {
 		applicationMenuItem = [NSMenuItem itemWithSubmenu:
-			[_defaultApplicationMenu copy]];
+			[[_defaultApplicationMenu copy] autorelease]];
 		[menu insertItem:applicationMenuItem atIndex:0];
 	    }
 	    [menu setSpecial:tkMainMenu];
@@ -420,7 +419,7 @@ static int	ModifierCharWidth(Tk_Font tkfont);
 	applicationMenu = (TKMenu *)[applicationMenuItem submenu];
 	if (![applicationMenu isSpecial:tkApplicationMenu]) {
 	    for (NSMenuItem *item in _defaultApplicationMenuItems) {
-		[applicationMenu addItem:[item copy]];
+		[applicationMenu addItem:[[item copy] autorelease]];
 	    }
 	    [applicationMenu setSpecial:tkApplicationMenu];
 	}
@@ -430,13 +429,15 @@ static int	ModifierCharWidth(Tk_Font tkfont);
 	for (NSMenuItem *item in itemArray) {
 	    TkMenuEntry *mePtr = (TkMenuEntry *)[item tag];
 	    TKMenu *submenu = (TKMenu *)[item submenu];
+
 	    if (mePtr && submenu) {
 		if ((mePtr->entryFlags & ENTRY_WINDOWS_MENU) &&
 			![submenu isSpecial:tkWindowsMenu]) {
 		    NSInteger index = 0;
 
 		    for (NSMenuItem *i in _defaultWindowsMenuItems) {
-			[submenu insertItem:[i copy] atIndex:index++];
+			[submenu insertItem:[[i copy] autorelease] atIndex:
+				index++];
 		    }
 		    [self setWindowsMenu:submenu];
 		    [submenu setSpecial:tkWindowsMenu];
@@ -445,7 +446,8 @@ static int	ModifierCharWidth(Tk_Font tkfont);
 		    NSInteger index = 0;
 
 		    for (NSMenuItem *i in _defaultHelpMenuItems) {
-			[submenu insertItem:[i copy] atIndex:index++];
+			[submenu insertItem:[[i copy] autorelease] atIndex:
+				index++];
 		    }
 		    [submenu setSpecial:tkHelpMenu];
 		}
@@ -494,7 +496,8 @@ TkpNewMenu(
 				 * platform structure for. */
 {
     TKMenu *menu = [[TKMenu alloc] initWithTkMenu:menuPtr];
-    menuPtr->platformData = (TkMenuPlatformData) menu;
+    menuPtr->platformData = (TkMenuPlatformData)
+	    TkMacOSXMakeUncollectable(menu);
     CheckForSpecialMenu(menuPtr);
     return TCL_OK;
 }
@@ -519,10 +522,7 @@ void
 TkpDestroyMenu(
     TkMenu *menuPtr)		/* The common menu structure */
 {
-    NSMenu* nsmenu = (NSMenu*)(menuPtr->platformData);
-
-    [nsmenu release];
-    menuPtr->platformData = NULL;
+    TkMacOSXMakeCollectableAndRelease(menuPtr->platformData);
 }
 
 /*
@@ -555,7 +555,8 @@ TkpMenuNewEntry(
     } else {
 	menuItem = [menu newTkMenuItem:mePtr];
     }
-    mePtr->platformEntryData = (TkMenuPlatformEntryData) menuItem;
+    mePtr->platformEntryData = (TkMenuPlatformEntryData)
+	    TkMacOSXMakeUncollectable(menuItem);
 
     /*
      * Caller TkMenuEntry() already did this same insertion into the generic
@@ -682,18 +683,18 @@ TkpConfigureMenuEntry(
 		  int i = 0;
 		  NSArray *itemArray = [submenu itemArray];
 		  for (NSMenuItem *item in itemArray) {
-		      TkMenuEntry *submePtr = menuRefPtr->menuPtr->entries[i];
-		        /* Work around an apparent bug where itemArray can have
+		    TkMenuEntry *submePtr = menuRefPtr->menuPtr->entries[i];
+		    /* Work around an apparent bug where itemArray can have
                       more items than the menu's entries[] array. */
                     if (i >= menuRefPtr->menuPtr->numEntries) break;
-		      [item setEnabled: !(submePtr->state == ENTRY_DISABLED)];
-		    i++;		      
+		    [item setEnabled: !(submePtr->state == ENTRY_DISABLED)];
+		    i++;
 		  }
 		}
+
 	    }
 	}
     }
- 
     [menuItem setSubmenu:submenu];
 
     return TCL_OK;
@@ -719,21 +720,16 @@ void
 TkpDestroyMenuEntry(
     TkMenuEntry *mePtr)
 {
-    NSMenuItem *menuItem;
-    TKMenu *menu;
-    NSInteger index;
-
     if (mePtr->platformEntryData && mePtr->menuPtr->platformData) {
-	menu = (TKMenu *) mePtr->menuPtr->platformData;
-	menuItem = (NSMenuItem *) mePtr->platformEntryData;
-	index = [menu indexOfItem:menuItem];
+	TKMenu *menu = (TKMenu *) mePtr->menuPtr->platformData;
+	NSMenuItem *menuItem = (NSMenuItem *) mePtr->platformEntryData;
+	NSInteger index = [menu indexOfItem:menuItem];
 
 	if (index > -1) {
 	    [menu removeItemAtIndex:index];
 	}
-	[menuItem release];
-	mePtr->platformEntryData = NULL;
     }
+    TkMacOSXMakeCollectableAndRelease(mePtr->platformEntryData);
 }
 
 /*
@@ -761,7 +757,7 @@ TkpPostMenu(
 				 * to be posted. */
     int y)			/* The global y-coordinate */
 {
- 
+
 
     /* Get the object that holds this Tk Window.*/
     Tk_Window root;
@@ -769,11 +765,11 @@ TkpPostMenu(
     if (root == NULL) {
 	return TCL_ERROR;
     }
- 
+
     Drawable d = Tk_WindowId(root);
     NSView *rootview = TkMacOSXGetRootControl(d);
     NSWindow *win = [rootview window];
- 
+
     inPostMenu = 1;
 
     int oldMode = Tcl_SetServiceMode(TCL_SERVICE_NONE);
@@ -781,7 +777,7 @@ TkpPostMenu(
     NSRect frame = NSMakeRect(x + 9, tkMacOSXZeroScreenHeight - y - 9, 1, 1);
 
     frame.origin = [view convertPoint:
-	    [win convertPointFromScreen:frame.origin] fromView:nil];
+	    [win convertScreenToBase:frame.origin] fromView:nil];
 
     NSMenu *menu = (NSMenu *) menuPtr->platformData;
     NSPopUpButtonCell *popUpButtonCell = [[NSPopUpButtonCell alloc]
@@ -834,25 +830,21 @@ TkpSetWindowMenuBar(
  *	Puts the menu associated with a window into the menubar. Should only
  *	be called when the window is in front.
  *
- *      This is a no-op on all other platforms.  On OS X it is a no-op when
- *      passed a NULL menuName or a nonexistent menuName, with an exception
- *      for the first call in a new interpreter.  In that special case, passing a
- *      NULL menuName installs the default menu.
- *
  * Results:
  *	None.
  *
  * Side effects:
- *	The menubar may be changed.
+ *	The menubar is changed.
  *
  *----------------------------------------------------------------------
  */
- 
+
 void
 TkpSetMainMenubar(
     Tcl_Interp *interp,		/* The interpreter of the application */
     Tk_Window tkwin,		/* The frame we are setting up */
-    char *menuName)	/* The name of the menu to put in front.*/
+    const char *menuName)	/* The name of the menu to put in front. If
+				 * NULL, use the default menu bar. */
 {
     static Tcl_Interp *currentInterp = NULL;
     TKMenu *menu = nil;
@@ -1462,10 +1454,10 @@ TkpMenuInit(void)
     [NSMenuItem setUsesUserKeyEquivalents:NO];
     tkColPtr = TkpGetColor(None, DEF_MENU_BG_COLOR);
     defaultBg = tkColPtr->color.pixel;
-    ckfree((char *) tkColPtr);
+    ckfree(tkColPtr);
     tkColPtr = TkpGetColor(None, DEF_MENU_FG);
     defaultFg = tkColPtr->color.pixel;
-    ckfree((char *) tkColPtr);
+    ckfree(tkColPtr);
 
     ChkErr(GetThemeMetric, kThemeMetricMenuMarkColumnWidth,
 	    &menuMarkColumnWidth);
@@ -1528,7 +1520,7 @@ TkpMenuThreadInit(void)
 void
 TkpMenuNotifyToplevelCreate(
     Tcl_Interp *interp,		/* The interp the menu lives in. */
-    char *menuName)	/* The name of the menu to reconfigure. */
+    const char *menuName)	/* The name of the menu to reconfigure. */
 {
     /*
      * Nothing to do.
