@@ -6,6 +6,8 @@
  *
  * Copyright 2001-2009, Apple Inc.
  * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright (c) 2015 Kevin Walzer/WordTech Communications LLC.
+ * Copyright (c) 2015 Marc Culler.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -356,11 +358,12 @@ GenerateUpdates(
     event.xexpose.width = damageBounds.size.width;
     event.xexpose.height = damageBounds.size.height;
     event.xexpose.count = 0;
-    Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
-#ifdef TK_MAC_DEBUG_DRAWING
-    TKLog(@"Expose %p {{%d, %d}, {%d, %d}}", event.xany.window, event.xexpose.x,
+    Tk_HandleEvent(&event);
+
+    #ifdef TK_MAC_DEBUG_DRAWING
+    NSLog(@"Expose %p {{%d, %d}, {%d, %d}}", event.xany.window, event.xexpose.x,
 	event.xexpose.y, event.xexpose.width, event.xexpose.height);
-#endif
+    #endif
 
     /*
      * Generate updates for the children of this window
@@ -759,15 +762,29 @@ Tk_MacOSXIsAppInFront(void)
 #import <ApplicationServices/ApplicationServices.h>
 
 /*
- * Custom content view for Tk NSWindows, containing standard NSView subviews.
- * The goal is to emulate X11-style drawing in response to Expose events:
- * during the normal AppKit drawing cycle, we supress drawing of all subviews
- * and instead send Expose events about the subviews that would be redrawn.
+ * Custom content view for use in Tk NSWindows.
+ * 
+ * Since Tk handles all drawing of widgets, we only use the AppKit event loop
+ * as a source of input events.  To do this, we overload the NSView drawRect
+ * method with a method which generates Expose events for Tk but does no
+ * drawing.  The redrawing operations are then done when Tk processes these
+ * events.
+ *
+ * Earlier versions of Mac Tk used subclasses of NSView, e.g. NSButton, as the
+ * basis for Tk widgets.  These would then appear as subviews of the
+ * TKContentView.  To prevent the AppKit from redrawing and corrupting the Tk
+ * Widgets it was necessary to use Apple private API calls.  In order to avoid
+ * using private API calls, the NSView-based widgets have been replaced with
+ * normal Tk widgets which draw themselves as native widgets by using the
+ * HITheme API.
+ *
  */
 
 @interface TKContentView(TKWindowEvent)
 - (void) drawRect: (NSRect) rect;
 - (void) generateExposeEvents: (HIMutableShapeRef) shape;
+- (void) viewDidEndLiveResize;
+- (void) tkToolbarButton: (id) sender;
 - (BOOL) isOpaque;
 - (BOOL) wantsDefaultClipping;
 - (BOOL) acceptsFirstResponder;
@@ -777,6 +794,8 @@ Tk_MacOSXIsAppInFront(void)
 @implementation TKContentView
 @end
 
+
+/*Restrict event processing to Expose events.*/
 static Tk_RestrictAction
 ExposeRestrictProc(
     ClientData arg,
@@ -794,6 +813,7 @@ ExposeRestrictProc(
     NSInteger rectsBeingDrawnCount;
 
     [self getRectsBeingDrawn:&rectsBeingDrawn count:&rectsBeingDrawnCount];
+
 #ifdef TK_MAC_DEBUG_DRAWING
     TKLog(@"-[%@(%p) %s%@]", [self class], self, _cmd, NSStringFromRect(rect));
     [[NSColor colorWithDeviceRed:0.0 green:1.0 blue:0.0 alpha:.1] setFill];
@@ -806,7 +826,6 @@ ExposeRestrictProc(
 
     while (rectsBeingDrawnCount--) {
 	CGRect r = NSRectToCGRect(*rectsBeingDrawn++);
-
 	r.origin.y = height - (r.origin.y + r.size.height);
 	HIShapeUnionWithRect(drawShape, &r);
     }
@@ -819,45 +838,72 @@ ExposeRestrictProc(
 			NSEventTrackingRunLoopMode, NSModalPanelRunLoopMode,
 			nil]];
     }
+
     CFRelease(drawShape);
 }
 
+/*
+ * As insurance against bugs that might cause layout glitches during a live
+ * resize, we redraw the window at the end of the resize operation.
+ */
+
+- (void)viewDidEndLiveResize
+{
+    HIRect bounds = NSRectToCGRect([self bounds]);
+    HIShapeRef shape = HIShapeCreateWithRect(&bounds);
+    [self generateExposeEvents: shape];
+
+}
+
+/*Core function of this class, generates expose events for redrawing.*/
 - (void) generateExposeEvents: (HIMutableShapeRef) shape
 {
+
     TkWindow *winPtr = TkMacOSXGetTkWindow([self window]);
     unsigned long serial;
     CGRect updateBounds;
 
     if (!winPtr) {
-	return;
+		return;
     }
+
+
     HIShapeGetBounds(shape, &updateBounds);
     serial = LastKnownRequestProcessed(Tk_Display(winPtr));
     if (GenerateUpdates(shape, &updateBounds, winPtr) &&
-	    ![[NSRunLoop currentRunLoop] currentMode] &&
-	    Tcl_GetServiceMode() != TCL_SERVICE_NONE) {
-	/*
-	 * Ensure there are no pending idle-time redraws that could prevent the
-	 * just posted Expose events from generating new redraws.
-	 */
+	![[NSRunLoop currentRunLoop] currentMode] &&
+	Tcl_GetServiceMode() != TCL_SERVICE_NONE) {
+    	/*
+    	 * Ensure there are no pending idle-time redraws that could
+         * prevent the just posted Expose events from generating
+         * new redraws.
+    	 */
 
 	while (Tcl_DoOneEvent(TCL_IDLE_EVENTS|TCL_DONT_WAIT)) {}
 
-	/*
-	 * For smoother drawing, process Expose events and resulting redraws
-	 * immediately instead of at idle time.
-	 */
+    	/*
+    	 * For smoother drawing, process Expose events and resulting
+         * redraws immediately instead of at idle time.
+    	 */
 
-	ClientData oldArg;
-	Tk_RestrictProc *oldProc = Tk_RestrictEvents(ExposeRestrictProc,
-		UINT2PTR(serial), &oldArg);
+    	ClientData oldArg;
+    	Tk_RestrictProc *oldProc = Tk_RestrictEvents(ExposeRestrictProc,
+						     UINT2PTR(serial), &oldArg);
+
+    	while (Tcl_ServiceEvent(TCL_WINDOW_EVENTS)) {}
+
+    	Tk_RestrictEvents(oldProc, oldArg, &oldArg);
+
+    	while (Tcl_DoOneEvent(TCL_IDLE_EVENTS|TCL_DONT_WAIT)) {}
 
-	while (Tcl_ServiceEvent(TCL_WINDOW_EVENTS)) {}
-	Tk_RestrictEvents(oldProc, oldArg, &oldArg);
-	while (Tcl_DoOneEvent(TCL_IDLE_EVENTS|TCL_DONT_WAIT)) {}
     }
+
 }
 
+/*
+ * This is no-op on 10.7 and up because Apple has removed this widget,
+ * but we are leaving it here for backwards compatibility.
+ */
 - (void) tkToolbarButton: (id) sender
 {
 #ifdef TK_MAC_DEBUG_EVENTS
@@ -885,22 +931,6 @@ ExposeRestrictProc(
     Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
 }
 
-#ifdef TK_MAC_DEBUG_DRAWING
-- (void) setFrameSize: (NSSize) newSize
-{
-    TKLog(@"-[%@(%p) %s%@]", [self class], self, _cmd,
-	    NSStringFromSize(newSize));
-    [super setFrameSize:newSize];
-}
-
-- (void) setNeedsDisplayInRect: (NSRect) invalidRect
-{
-    TKLog(@"-[%@(%p) %s%@]", [self class], self, _cmd,
-	    NSStringFromRect(invalidRect));
-    [super setNeedsDisplayInRect:invalidRect];
-}
-#endif
-
 - (BOOL) isOpaque
 {
     NSWindow *w = [self window];
